(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     15160,        540]
NotebookOptionsPosition[     11964,        424]
NotebookOutlinePosition[     12431,        445]
CellTagsIndexPosition[     12388,        442]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["PwM`Chap06Procedural`", "Section",
 InitializationGroup->True],

Cell[TextData[{
 StyleBox["From",
  FontWeight->"Bold"],
 ": ",
 StyleBox[ButtonBox["Programming with Mathematica: An Introduction",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.cambridge.org/wellin"], None},
  ButtonNote->"http://www.cambridge.org/wellin"],
  FontSlant->"Italic",
  FontVariations->{"Underline"->True}]
}], "Text",
 CellMargins->{{28, 10}, {7, 7}},
 FontSize->14],

Cell[CellGroupData[{

Cell["Metadata", "Subsection"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{":", 
   RowBox[{"Name", ":", " ", "PwM`Chap06Procedural`"}]}], " ", 
  "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{":", 
   RowBox[{"Book", ":", " ", 
    RowBox[{"Programming", " ", "with", " ", 
     RowBox[{"Mathematica", ":", " ", 
      RowBox[{"An", " ", "Introduction"}]}]}]}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{":", 
   RowBox[{"Author", ":", " ", 
    RowBox[{"Paul", " ", "Wellin"}]}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{":", 
    RowBox[{"Summary", ":", " ", 
     RowBox[{
     "This", " ", "package", " ", "contains", " ", "the", " ", "functions", 
      " ", "defined", " ", "in", " ", "Chapter", " ", "6"}]}]}], ",", " ", 
   RowBox[{"Procedural", " ", 
    RowBox[{"programming", "."}]}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{":", 
   RowBox[{"Context", ":", " ", "PwM`Chap06Procedural`"}]}], " ", 
  "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{":", 
   RowBox[{"Package", " ", 
    RowBox[{"Version", ":", " ", "1.0"}]}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{":", "Copyright", ":", " ", 
    RowBox[{"Copyright", " ", "2012"}]}], ",", " ", 
   RowBox[{"Paul", " ", 
    RowBox[{"R", ".", " ", "Wellin"}]}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{":", 
    RowBox[{"History", ":", " ", 
     RowBox[{
     "Version", " ", "1.0", " ", "by", " ", "Paul", " ", "Wellin"}]}]}], ",", 
   " ", 
   RowBox[{"Dec", " ", "2012"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{":", 
    RowBox[{"Keywords", ":", " ", 
     RowBox[{"digit", " ", "root"}]}]}], ",", " ", 
   RowBox[{"lucky", " ", "number"}], ",", " ", 
   RowBox[{"upper", " ", "triangular", " ", "matrix"}]}], " ", 
  "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{":", 
   RowBox[{"Source", ":", " ", 
    RowBox[{"Chap06Procedural", ".", "nb"}]}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{":", 
   RowBox[{"Mathematica", " ", 
    RowBox[{"Version", ":", " ", 
     RowBox[{"8.0", ".4"}]}]}]}], " ", "*)"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["BeginPackage", "Subsection"],

Cell[BoxData[
 RowBox[{"Remove", "[", 
  RowBox[{"DigitRoot", ",", "UpperTriangularMatrix"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"BeginPackage", "[", "\"\<PwM`Chap06Procedural`\>\"", "]"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Usage messages for public functions", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DigitRoot", "::", "usage"}], "=", 
   "\"\<DigitRoot[n] returns the digital root of the integer n.\>\""}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LuckyNumbers", "::", "usage"}], "=", 
   "\"\<LuckyNumber[n] returns a list of the lucky numbers less than \
n.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"UpperTriangularMatrix", "::", "usage"}], "=", 
   "\"\<UpperTriangularMatrix[n] returns a square matrix of size n with zeros \
below the diagonal and ones on and above the diagonal.\>\""}], ";"}]], "Input"]
}, Closed]],

Cell["Options", "Subsection"],

Cell[CellGroupData[{

Cell["Private context", "Subsection"],

Cell[BoxData[
 RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Function definitions (public and private)", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DigitRoot", "[", 
   RowBox[{"n_Integer", "?", "Positive"}], "]"}], ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"n", ",", "9"}], "]"}], "\[Equal]", "0"}], ",", "9", ",", 
    RowBox[{"Mod", "[", 
     RowBox[{"n", ",", "9"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"LuckyNumbers", "[", 
   RowBox[{"n_Integer", "?", "Positive"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", 
      RowBox[{"i", "=", "2"}], ",", 
      RowBox[{"ints", "=", 
       RowBox[{"Range", "[", 
        RowBox[{"1", ",", "n", ",", "2"}], "]"}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"ints", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "<", 
        RowBox[{"Length", "[", "ints", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"p", "=", 
         RowBox[{"ints", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"ints", "=", 
         RowBox[{"Drop", "[", 
          RowBox[{"ints", ",", 
           RowBox[{"p", ";;", 
            RowBox[{"-", "1"}], ";;", "p"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "++"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     "ints"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"UpperTriangularMatrix", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"m_", ",", "n_"}], "}"}], ",", 
    RowBox[{"val_:", "1"}]}], "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"i", "\[GreaterEqual]", "j"}], ",", "0", ",", "val"}], "]"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "m"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "n"}], "}"}]}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["End private context", "Subsection"],

Cell[BoxData[
 RowBox[{"End", "[", "]"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["EndPackage", "Subsection"],

Cell[BoxData[
 RowBox[{"EndPackage", "[", "]"}]], "Input"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Section"],

Cell[CellGroupData[{

Cell["Load package", "Subsection"],

Cell["Quit current session:", "Text"],

Cell[BoxData["Quit"], "Input"],

Cell[TextData[{
 "Load package (do ",
 StyleBox["not",
  FontSlant->"Italic"],
 " say \[OpenCurlyDoubleQuote]Yes\[CloseCurlyDoubleQuote] to evaluate init \
cells dialog)."
}], "Text"],

Cell[BoxData[
 RowBox[{"<<", "PWM`Chap06Procedural`"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Test usage messages", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "DigitRoot"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"DigitRoot[\!\(\*\\nStyleBox[\\\"n\\\", \\\"TI\\\"]\)] returns \
the digital root of the integer \!\(\*\\nStyleBox[\\\"n\\\", \\\"TI\\\"]\).\"\
\>", "MSG"]], "Print", "PrintUsage"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "LuckyNumbers"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"LuckyNumber[\!\(\*\\nStyleBox[\\\"n\\\", \\\"TI\\\"]\)] \
returns a list of the lucky numbers less than \!\(\*\\nStyleBox[\\\"n\\\", \\\
\"TI\\\"]\).\"\>", "MSG"]], "Print", "PrintUsage"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "UpperTriangularMatrix"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"UpperTriangularMatrix[\\!\\(\\*\\nStyleBox[\\\"n\\\", \\\"TI\\\
\"]\\)] returns a square matrix of size \\!\\(\\*\\nStyleBox[\\\"n\\\", \
\\\"TI\\\"]\\) with zeros below the diagonal and ones on and above the \
diagonal.\"\>", "MSG"]], "Print", "PrintUsage"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercise the functions and their options", "Subsection"],

Cell[CellGroupData[{

Cell["DigitRoot", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DigitRoot", "[", 
  RowBox[{"1000", "!"}], "]"}]], "Input"],

Cell[BoxData["9"], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LuckyNumbers", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LuckyNumbers", "[", "1000", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "3", ",", "7", ",", "9", ",", "13", ",", "15", ",", "21", ",", 
   "25", ",", "31", ",", "33", ",", "37", ",", "43", ",", "49", ",", "51", 
   ",", "63", ",", "67", ",", "69", ",", "73", ",", "75", ",", "79", ",", 
   "87", ",", "93", ",", "99", ",", "105", ",", "111", ",", "115", ",", "127",
    ",", "129", ",", "133", ",", "135", ",", "141", ",", "151", ",", "159", 
   ",", "163", ",", "169", ",", "171", ",", "189", ",", "193", ",", "195", 
   ",", "201", ",", "205", ",", "211", ",", "219", ",", "223", ",", "231", 
   ",", "235", ",", "237", ",", "241", ",", "259", ",", "261", ",", "267", 
   ",", "273", ",", "283", ",", "285", ",", "289", ",", "297", ",", "303", 
   ",", "307", ",", "319", ",", "321", ",", "327", ",", "331", ",", "339", 
   ",", "349", ",", "357", ",", "361", ",", "367", ",", "385", ",", "391", 
   ",", "393", ",", "399", ",", "409", ",", "415", ",", "421", ",", "427", 
   ",", "429", ",", "433", ",", "451", ",", "463", ",", "475", ",", "477", 
   ",", "483", ",", "487", ",", "489", ",", "495", ",", "511", ",", "517", 
   ",", "519", ",", "529", ",", "535", ",", "537", ",", "541", ",", "553", 
   ",", "559", ",", "577", ",", "579", ",", "583", ",", "591", ",", "601", 
   ",", "613", ",", "615", ",", "619", ",", "621", ",", "631", ",", "639", 
   ",", "643", ",", "645", ",", "651", ",", "655", ",", "673", ",", "679", 
   ",", "685", ",", "693", ",", "699", ",", "717", ",", "723", ",", "727", 
   ",", "729", ",", "735", ",", "739", ",", "741", ",", "745", ",", "769", 
   ",", "777", ",", "781", ",", "787", ",", "801", ",", "805", ",", "819", 
   ",", "823", ",", "831", ",", "841", ",", "855", ",", "867", ",", "873", 
   ",", "883", ",", "885", ",", "895", ",", "897", ",", "903", ",", "925", 
   ",", "927", ",", "931", ",", "933", ",", "937", ",", "957", ",", "961", 
   ",", "975", ",", "979", ",", "981", ",", "991", ",", "993", ",", "997"}], 
  "}"}]], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["UpperTriangularMatrix", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"UpperTriangularMatrix", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"5", ",", "5"}], "}"}], ",", "\[Alpha]"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "\[Alpha]", "\[Alpha]", "\[Alpha]", "\[Alpha]"},
     {"0", "0", "\[Alpha]", "\[Alpha]", "\[Alpha]"},
     {"0", "0", "0", "\[Alpha]", "\[Alpha]"},
     {"0", "0", "0", "0", "\[Alpha]"},
     {"0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Closed]]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
ScreenStyleEnvironment->"Working",
WindowSize->{
  Scaled[0.5], 
  Scaled[0.75]},
WindowMargins->{{15, Automatic}, {Automatic, 15}},
ShowSelection->True,
FrontEndVersion->"10.1 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 23, \
2015)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 68, 1, 60, "Section",
 InitializationGroup->True],
Cell[650, 25, 398, 13, 29, "Text"],
Cell[CellGroupData[{
Cell[1073, 42, 30, 0, 41, "Subsection"],
Cell[1106, 44, 134, 4, 36, "Input"],
Cell[1243, 50, 239, 6, 36, "Input"],
Cell[1485, 58, 146, 4, 35, "Input"],
Cell[1634, 64, 355, 9, 78, "Input"],
Cell[1992, 75, 137, 4, 36, "Input"],
Cell[2132, 81, 147, 4, 36, "Input"],
Cell[2282, 87, 229, 6, 36, "Input"],
Cell[2514, 95, 256, 8, 57, "Input"],
Cell[2773, 105, 281, 8, 57, "Input"],
Cell[3057, 115, 154, 4, 36, "Input"],
Cell[3214, 121, 173, 5, 35, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[3424, 131, 34, 0, 33, "Subsection"],
Cell[3461, 133, 110, 2, 51, "Input"],
Cell[3574, 137, 93, 1, 51, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[3704, 143, 57, 0, 33, "Subsection"],
Cell[3764, 145, 173, 5, 28, "Input"],
Cell[3940, 152, 183, 5, 46, "Input"],
Cell[4126, 159, 253, 5, 63, "Input"]
}, Closed]],
Cell[4394, 167, 29, 0, 33, "Subsection"],
Cell[CellGroupData[{
Cell[4448, 171, 37, 0, 33, "Subsection"],
Cell[4488, 173, 74, 1, 97, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[4599, 179, 63, 0, 33, "Subsection"],
Cell[4665, 181, 338, 10, 51, "Input"],
Cell[5006, 193, 1078, 31, 231, "Input"],
Cell[6087, 226, 482, 16, 82, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[6606, 247, 41, 0, 33, "Subsection"],
Cell[6650, 249, 51, 1, 97, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[6738, 255, 32, 0, 33, "Subsection"],
Cell[6773, 257, 58, 1, 97, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6880, 264, 27, 0, 60, "Section"],
Cell[CellGroupData[{
Cell[6932, 268, 34, 0, 41, "Subsection"],
Cell[6969, 270, 37, 0, 96, "Text"],
Cell[7009, 272, 30, 0, 97, "Input"],
Cell[7042, 274, 183, 6, 96, "Text"],
Cell[7228, 282, 65, 1, 97, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[7330, 288, 41, 0, 33, "Subsection"],
Cell[CellGroupData[{
Cell[7396, 292, 52, 1, 63, "Input"],
Cell[7451, 295, 208, 3, 147, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[7696, 303, 55, 1, 47, "Input"],
Cell[7754, 306, 216, 3, 147, "Print"]
}, Closed]],
Cell[CellGroupData[{
Cell[8007, 314, 64, 1, 47, "Input"],
Cell[8074, 317, 287, 4, 210, "Print"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[8410, 327, 62, 0, 33, "Subsection"],
Cell[CellGroupData[{
Cell[8497, 331, 34, 0, 32, "Subsubsection"],
Cell[CellGroupData[{
Cell[8556, 335, 83, 2, 97, "Input"],
Cell[8642, 339, 28, 0, 97, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[8719, 345, 37, 0, 26, "Subsubsection"],
Cell[CellGroupData[{
Cell[8781, 349, 68, 1, 97, "Input"],
Cell[8852, 352, 1964, 28, 737, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[10865, 386, 46, 0, 26, "Subsubsection"],
Cell[CellGroupData[{
Cell[10936, 390, 192, 6, 97, "Input"],
Cell[11131, 398, 781, 20, 331, "Output"]
}, Closed]]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
